function [ b_GLRT_opt, GLRT_val ] = BistaticBFSKNonoherentSequenceDetection( y_GLRT ) %Algorithm 1

%length of the sequence
[~,N_seq] = size(y_GLRT);

%the 2Nseq points of decision change
phis = zeros(1,2*N_seq);

%built the 2Nseq phases
for n = 1 : N_seq
    phis(n) = pi/2 + angle(y_GLRT(1,n) - y_GLRT(2,n));
    phis(N_seq + n)   = -pi/2 + angle(y_GLRT(1,n) - y_GLRT(2,n));
end
% phis
phis = mod(phis,2* pi);
%sort the phis
[theta, sorted_inds] = sort(phis);

% theta
%normalize sorted_inds in the range {1,2,..,N_seq}
sorted_inds = mod(sorted_inds-1,N_seq)+1;

%initialize decision sequence
b_hat = zeros(1,N_seq);

%find the optimal sequence at phi = 0;
for n = 1 : N_seq
    [~, ind_max] = max(real(y_GLRT(:,n)));
    b_hat(n) = ind_max;
end

complex_objective_val = 0;
for i = 1 : N_seq
%evaluate the objective for the current optimal sequence
complex_objective_val = complex_objective_val + y_GLRT(b_hat(i),i);
end
% y_GLRT
% y_GLRT(b_hat,:)
%store the corresponding metric value
objective_val = abs(complex_objective_val);

%up-to-date GLRT-optimal objective value
GLRT_val = objective_val;

%GLRT-optimal sequence
b_GLRT_opt = b_hat - 1;

for i = 1 : length(theta)-1
    %current index associated with the decision change
    cur_n = sorted_inds(i);
    
    %current optimal decision of the n-th index
    b_tmp = b_hat(cur_n);
    
    %update the best complex value
    complex_objective_val = complex_objective_val - y_GLRT(b_tmp,cur_n) + y_GLRT(3-b_tmp,cur_n);
    
    %update the best value for the objective
    objective_val = abs(complex_objective_val);
    
    %apply decision change
    b_hat(cur_n) = 3 - b_tmp;
    
    if objective_val > GLRT_val
        %up-to-date GLRT-optimal objective value
        GLRT_val = objective_val;
        
        %GLRT-optimal sequence
        b_GLRT_opt = b_hat - 1;
    end
end
end

